﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

public class RenderSettingsWindow : EditorWindow {

		private static HashSet<IRenderSettingPreset> presets = new HashSet<IRenderSettingPreset>();

		private static Dictionary<IRenderSettingPreset, bool> presetCollapsedDict = new Dictionary<IRenderSettingPreset, bool>();

		private IRenderSettingPresetBuilder presetBuilder = new RenderSettingPresetBuilder();

		private RenderSettingApplier settingApplier = new RenderSettingApplier();
		private ISettingsSaver settingsSaver = new SettingsSaver();
		private SettingsLoader settingsLoader = new SettingsLoader();

		private const bool isNotCollapsed = true;

		Vector2 scrollPosition;

		private bool isCreatingVisualBlueprint = false;
		private IRenderSettingPreset focusedPreset;

		private bool isCreatingPreset = false;

		void OnGUI() {
			scrollPosition = EditorGUILayout.BeginScrollView(scrollPosition);
			if (presets.Count == 0) {
				EditorGUILayout.LabelField("No presets available");
			} else {
				foreach (IRenderSettingPreset preset in presets) {
					DrawPreset(preset);
				}
			}

			if(isCreatingPreset) {
				DrawPresetCreator();
			} else if(GUILayout.Button("Create new preset")) {
				isCreatingPreset = true;
			} else if (GUILayout.Button("Load preset")) {
				IRenderSettingPreset preset = settingsLoader.Load();
				presets.Add(preset);
				presetCollapsedDict.Add(preset, isNotCollapsed);
			}

			EditorGUILayout.EndScrollView();
		}

		private string presetName;
		private void DrawPresetCreator() {

		GUILayout.BeginHorizontal();

			const string nameLabel = "Name: ";
			EditorGUIUtility.labelWidth = CalcSmallestLabelWidth(nameLabel);
			presetName = EditorGUILayout.TextField(nameLabel, presetName);
			EditorGUIUtility.labelWidth = 0;

			bool createButtonPressed = GUILayout.Button("Create", GUILayout.Width(100));

			bool nameInUse = CheckPresetNameInUse(presetName);

			if (createButtonPressed) {
				bool validName = !nameInUse && !string.IsNullOrEmpty(presetName);
				if (validName) {
					IRenderSettingPreset preset = presetBuilder.Build(presetName);
					presets.Add(preset);
					presetCollapsedDict.Add(preset, isNotCollapsed);
					isCreatingPreset = false;
					presetName = null;
				}
			}

		GUILayout.EndHorizontal();

		DisplayCreatePresetErrorMessages(nameInUse, presetName);

		}

		private bool CheckPresetNameInUse(string name) {
			foreach (IRenderSettingPreset p in presets) {
				if (p.GetName().Equals(name)) {
					return true;
				}
			}
			return false;
		}

		private void DisplayCreatePresetErrorMessages(bool nameInUse, string name) {
			if (nameInUse) {
				EditorGUILayout.LabelField("Error Name \'" + name + "\' already in use for a preset");
			}

			if (string.IsNullOrEmpty(name)) {
				EditorGUILayout.LabelField("Error Name cannot be empty");
			}
		}

		private void RemovePreset(object o) {
			presets.Remove((IRenderSettingPreset)o);
		}

		private void DrawPreset(IRenderSettingPreset preset) {

			GUILayout.BeginHorizontal();
			bool foldoutOpen = EditorGUILayout.Foldout(presetCollapsedDict[preset],
															preset.GetName(),
															EditorStyles.foldout);

			GUILayout.FlexibleSpace();

			if (preset.GetVisualBlueprints().Count > 0) {
				if (GUILayout.Button("Apply All", GUILayout.Width(100))) {
					settingApplier.Apply(preset);
				}

				if (GUILayout.Button("Save", GUILayout.Width(100))) {
					SavePreset(preset);
				}
			}

			GUILayout.FlexibleSpace();


			if (AddDropdownSettingsButton(FocusType.Keyboard)) {
				GenericMenu presetMenu = new GenericMenu();
				presetMenu.AddItem(new GUIContent("Remove Preset"), false, RemovePreset, preset);
				presetMenu.ShowAsContext();
			}
			GUILayout.EndHorizontal();


			presetCollapsedDict[preset] = foldoutOpen;

			if (foldoutOpen) {
				EditorGUI.indentLevel++;

				foreach (VisualBlueprint blueprint in preset.GetVisualBlueprints()) {
					DrawVisualBlueprint(blueprint, preset);
				}
				EditorGUI.indentLevel--;

				if (isCreatingVisualBlueprint && focusedPreset == preset) {
					DrawVisualBlueprintCreator();
				} else {
					GUILayout.BeginHorizontal();
					GUILayout.FlexibleSpace();
					if (GUILayout.Button("Add Blueprint", GUILayout.Width(150))) {
						isCreatingVisualBlueprint = true;
						focusedPreset = preset;
					}
					GUILayout.FlexibleSpace();
					GUILayout.EndHorizontal();
				}
			}
		}

		private void SavePreset(IRenderSettingPreset preset) {
			settingsSaver.Save(preset);
		}

		private bool AddDropdownSettingsButton(FocusType type) {
			GUIStyle iconButtonStyle = GUI.skin.FindStyle("IconButton") ?? EditorGUIUtility.GetBuiltinSkin(EditorSkin.Inspector).FindStyle("IconButton");
			GUIContent content = new GUIContent(EditorGUIUtility.Load("icons/_Popup.png") as Texture2D);

			return EditorGUILayout.DropdownButton(content, FocusType.Keyboard, iconButtonStyle);
		}

		private void DrawVisualBlueprint(VisualBlueprint blueprint, IRenderSettingPreset preset) {
			GUILayout.BeginHorizontal();

			EditorGUILayout.LabelField(blueprint.GetName());
			GameObject o = (GameObject) EditorGUILayout.ObjectField(blueprint.GetBlueprintObject(), typeof(GameObject), false);
			blueprint.SetBlueprintObject(o);

			if (GUILayout.Button("Apply", GUILayout.Width(50))) {
					settingApplier.Apply(blueprint);
			}

			GUILayout.FlexibleSpace();

			if (AddDropdownSettingsButton(FocusType.Keyboard)) {
				GenericMenu presetMenu = new GenericMenu();
				focusedPreset = preset; // Need to put the preset into a field because it is not possible to pass 2 arguments to a Menu Item callback method.
				presetMenu.AddItem(new GUIContent("Remove Blueprint"), false, RemoveVisualBlueprint, blueprint);
				presetMenu.ShowAsContext();
			}

			GUILayout.EndHorizontal();
		}

		private void RemoveVisualBlueprint(object blueprint) {
			focusedPreset.RemoveVisualBlueprint((VisualBlueprint)blueprint);
		}

		/**
			Calculates the smallest width for a label that allows the given message to be displayed
		**/
		private float CalcSmallestLabelWidth(string message) {
				return GUI.skin.label.CalcSize(new GUIContent(message)).x;
		}

		private string blueprintName;
		private GameObject blueprintObject;
		private void DrawVisualBlueprintCreator() {
			GUILayout.BeginHorizontal();

				const string nameLabel = "Name: ";
				const string templateLabel = "Template: ";
				EditorGUIUtility.labelWidth = CalcSmallestLabelWidth(nameLabel);
				blueprintName = EditorGUILayout.TextField(nameLabel, blueprintName);
				EditorGUIUtility.labelWidth = CalcSmallestLabelWidth(templateLabel);
				blueprintObject = (GameObject)EditorGUILayout.ObjectField(templateLabel , blueprintObject, typeof(GameObject), false);
				EditorGUIUtility.labelWidth = 0; //Reset the label width to its default size

				bool createButtonPressed = GUILayout.Button("Create", GUILayout.Width(100));


				bool nameInUse = CheckBlueprintNameInUse(blueprintName);
				bool validObject = blueprintObject != null;

				if (createButtonPressed) {
					bool validName = !nameInUse && !string.IsNullOrEmpty(blueprintName);
					if (validName && validObject) {
						focusedPreset.AddVisualBlueprint(new VisualBlueprint(blueprintName, blueprintObject));
						isCreatingVisualBlueprint = false;
						blueprintName = null;
						blueprintObject = null;
					}
				}

			GUILayout.EndHorizontal();

			DisplayCreateBlueprintErrorMessages(nameInUse, blueprintName, validObject);
		}

		private void DisplayCreateBlueprintErrorMessages(bool nameInUse, string name, bool validObject) {
			if (nameInUse) {
				EditorGUILayout.LabelField("Error Name \'" + name + "\' already in use in this preset");
			}

			if (string.IsNullOrEmpty(name)) {
				EditorGUILayout.LabelField("Error Name cannot be empty");
			}

			if (!validObject) {
				EditorGUILayout.LabelField("Error: Game Object cannot be None. You need to set a game object");
			}
		}

		private bool CheckBlueprintNameInUse(string name) {
			foreach (VisualBlueprint blueprint in focusedPreset.GetVisualBlueprints()) {
				if (blueprint.GetName().Equals(name)) {
					return true;
				}
			}
			return false;
		}
}
